/*
  Java gfx-test framework

   Copyright (C) 2010, 2011, 2012, 2013  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.gfxtest.framework;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.print.PrinterJob;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;



import javax.print.Doc;
import javax.print.DocFlavor;
import javax.print.DocPrintJob;
import javax.print.PrintException;
import javax.print.SimpleDoc;
import javax.print.StreamPrintService;
import javax.print.StreamPrintServiceFactory;
import javax.print.attribute.HashPrintRequestAttributeSet;
import javax.print.attribute.PrintRequestAttributeSet;
import javax.print.attribute.standard.Copies;
import javax.print.attribute.standard.MediaSizeName;

import org.gfxtest.common.ConfigurationException;
import org.gfxtest.framework.annotations.TestType;
import org.gfxtest.framework.annotations.Zoom;



/**
 * Base class which should be extended by each test suite. Only methods which
 * name begins with "test..." are run as test methods.
 * 
 * @author Pavel Tisnovsky
 */
public abstract class GfxTest
{
    /**
     * Simple logger that writes everything out to standard output and can use
     * color codes for ANSI-compatible terminals.
     */
    protected Log log = null;

    /**
     * Name of test suite.
     */
    protected String suiteName = null;

    /**
     * Size of cross used to mark end points, control points and other
     * interesting points in test image.
     */
    protected static final int CROSS_SIZE = 20;
    protected int currentTestNumber = 0;
    private int arcStep = ARC_STEP;

    /**
     * Minor radius for circles, ellipses etc.
     */
    protected static final int MINOR_RADIUS = 10;

    /**
     * Major radius for circles, ellipses etc.
     */
    protected static final int MAJOR_RADIUS = 100;

    /**
     * Changes in radius while drawing circles or ellipses.
     */
    protected static final int RADIUS_STEP = 30;

    /**
     * Number of angles used for rendering color wheels and similar test images.
     */
    protected static final int ANGLES = 16;

    /**
     * Offset from image borders used for rendering various graphics entities, for example rectangles.
     */
    public static final int OFFSET = 80;

    /**
     * Step between two arc radiuses.
     */
    protected static final int ARC_STEP = 10;

    /**
     * Default cap style.
     */
    static final int[] DEFAULT_CAP_STYLE = {BasicStroke.CAP_ROUND};

    /**
     * All cap styles available.
     */
    static final int[] ALL_CAP_STYLES = { BasicStroke.CAP_BUTT, BasicStroke.CAP_ROUND, BasicStroke.CAP_SQUARE };

    /**
     * Default join style.
     */
    static final int[] DEFAULT_JOIN_STYLE = {BasicStroke.JOIN_ROUND};

    /**
     * All available join styles.
     */
    static final int[] ALL_JOIN_STYLES = { BasicStroke.JOIN_BEVEL, BasicStroke.JOIN_MITER, BasicStroke.CAP_ROUND };

    /**
     * Default line or curve width.
     */
    static final float[] DEFAULT_WIDTH = { 1.0f };

    /**
     * All line or curve widths used by test cases.
     */
    static final float[] ALL_WIDTHS = { 1.0f, 2.0f, 5.0f, 10.0f, 20.0f, 40.0f };

    /**
     * Default scale.
     */
    static final float[] DEFAULT_SCALE = { 1.0f };

    /**
     * All scales used by test cases.
     */
    static final float[] ALL_SCALES = { 0.5f, 1.0f, 10.0f, 20.0f, 40.0f };

    /**
     * Default ellipse eccentricity.
     */
    static final float[] DEFAULT_ECCENTRICITY = { 1.0f };

    /**
     * Values used for rendering ellipses and similar geometric shapes.
     */
    //protected static final float[] ECCENTRICITIES = {1/3.0f, 1/2.5f, 1/2.0f, 1/1.5f, 1.0f, 1.5f, 2.0f, 2.5f, 3.0f};
    static final float[] ECCENTRICITIES = {1/3.0f, 1/1.5f, 1.0f, 1.5f, 3.0f};

    /**
     * Default pattern.
     */
    static final float[] DEFAULT_DASH_PATTERN = { 1.0f };

    /**
     * All dash patterns.
     */
    static final float[] ALL_DASH_PATTERNS = {1.0f, /*2.0f, */8.0f, 16.0f};

    /**
     * Write test suite duration to the log file.
     * 
     * @param t1
     *            time when the test suite has been started
     * @param t2
     *            time when the test suite was completed
     */
    private void printDuration(long t1, long t2)
    {
        this.log.logSet("duration", Long.valueOf(t2-t1)); //$NON-NLS-1$
    }

    /**
     * Construct test suite name using full class name.
     */
    private void fillSuiteName()
    {
        String className = this.getClass().getName();
        this.suiteName = className.substring(1 + className.lastIndexOf('.'));
    }

    protected void runTestSuite(String[] args)
    {
        long t1 = System.currentTimeMillis();
        this.log = new Log(this.getClass().getName(), false);
        this.log.logBegin("testsuite"); //$NON-NLS-1$
        fillSuiteName();
        GfxTestConfiguration configuration;
        try
        {
            configuration = new GfxTestConfiguration(args, this.log);
            runAllTests(configuration);
        }
        catch (ConfigurationException e)
        {
            e.printStackTrace();
        }
        finally
        {
            long t2 = System.currentTimeMillis();
            this.log.logEnd("testsuite"); //$NON-NLS-1$
            printDuration(t1, t2);
        }
    }

    /**
     * Invoke all methods which names start with "test" prefix. These methods
     * should accept one parameter of type Buffered image.
     *
     * @param configuration
     * @throws IOException
     * @throws IllegalArgumentException
     * @throws IllegalAccessException
     * @throws InvocationTargetException
     */
    private void runAllTests(GfxTestConfiguration configuration)
    {
        Class<? extends GfxTest> testClass = getClass();
        this.log.logBegin("common tests"); //$NON-NLS-1$
        Method[] methods = testClass.getDeclaredMethods();
        TestType testType = testClass.getAnnotation(TestType.class);
        for (Method method : methods)
        {
            tryToInvokeTestMethod(testType, configuration, method);
        }
        this.log.logEnd("common tests"); //$NON-NLS-1$
        this.log.logBegin("other tests"); //$NON-NLS-1$
        runOtherTests(configuration);
        this.log.logEnd("other tests"); //$NON-NLS-1$
    }

    /**
     * Method used for starts various tests which can be overridden by inherited
     * classes.
     *
     * @param configuration
     *            configuration of current test case.
     */
    protected void runOtherTests(GfxTestConfiguration configuration)
    {
        // to be overrided by test cases
    }
    
    /**
     * Method which can be overridden by inherited classes.
     * 
     * @param image
     *            image representing drawing destination
     * @param graphics
     *            graphics canvas derived from image object
     * @param testNumber
     *            test number
     * @param entityRenderingStyle
     *            object containing entity rendering style
     */
    protected void drawEntity(TestImage image, Graphics2D graphics, int testNumber, EntityRenderingStyle entityRenderingStyle)
    {
        // to be overrided by test cases
    }

    protected void drawEntityWithVariousStyles(GfxTestConfiguration configuration, boolean changeCap, boolean changeJoin, boolean changeWidth, boolean changeScale, boolean changeDash)
    {
        this.drawEntityWithVariousStyles(configuration, changeCap, changeJoin, changeWidth, changeScale, changeDash, 0, 0, 0, 0, false);
    }

    protected void drawEntityWithVariousStyles(GfxTestConfiguration configuration, boolean changeCap, boolean changeJoin, boolean changeWidth, boolean changeScale, boolean changeDash, boolean changeEccentricity)
    {
        this.drawEntityWithVariousStyles(configuration, changeCap, changeJoin, changeWidth, changeScale, changeDash, 0, 0, 0, 0, changeEccentricity);
    }

    protected void drawEntityWithVariousStyles(GfxTestConfiguration configuration, boolean changeCap, boolean changeJoin, boolean changeWidth, boolean changeScale, boolean changeDash, int minArcWidth, int maxArcWidth, int minArcHeight, int maxArcHeight)
    {
        this.drawEntityWithVariousStyles(configuration, changeCap, changeJoin, changeWidth, changeScale, changeDash, minArcWidth, maxArcWidth, minArcHeight, maxArcHeight, false);
    }

    protected void drawEntityWithVariousStyles(GfxTestConfiguration configuration, boolean changeCap, boolean changeJoin, boolean changeWidth, boolean changeScale, boolean changeDash, int minArcWidth, int maxArcWidth, int minArcHeight, int maxArcHeight, boolean changeEccentricity)
    {
        int[] caps  =    changeCap ? ALL_CAP_STYLES : DEFAULT_CAP_STYLE; 
        int[] joins =    changeJoin ? ALL_JOIN_STYLES : DEFAULT_JOIN_STYLE;
        float[] scales = changeScale ? ALL_SCALES : DEFAULT_SCALE;
        float[] widths = changeWidth ? ALL_WIDTHS : DEFAULT_WIDTH;
        float[] eccentricities = changeEccentricity ? ECCENTRICITIES : DEFAULT_ECCENTRICITY;
        float[] dash1patterns = changeDash ? ALL_DASH_PATTERNS : DEFAULT_DASH_PATTERN;
        float[] dash2patterns = changeDash ? ALL_DASH_PATTERNS : DEFAULT_DASH_PATTERN;

        this.currentTestNumber = 0;
        drawEntityUsingVariousCapStyles(configuration, minArcWidth, maxArcWidth, minArcHeight, maxArcHeight, caps, joins,
                scales, widths, eccentricities, dash1patterns, dash2patterns);
    }

    private void drawEntityUsingVariousCapStyles(GfxTestConfiguration configuration, int minArcWidth, int maxArcWidth,
            int minArcHeight, int maxArcHeight, int[] caps, int[] joins, float[] scales, float[] widths,
            float[] eccentricities, float[] dash1patterns, float[] dash2patterns)
    {
        for (int cap : caps)
        {
            drawEntityUsingVariousJoinStyles(configuration, minArcWidth, maxArcWidth, minArcHeight, maxArcHeight, joins,
                    scales, widths, eccentricities, dash1patterns, dash2patterns, cap);
        }
    }

    private void drawEntityUsingVariousJoinStyles(GfxTestConfiguration configuration, int minArcWidth, int maxArcWidth,
            int minArcHeight, int maxArcHeight, int[] joins, float[] scales, float[] widths, float[] eccentricities,
            float[] dash1patterns, float[] dash2patterns, int cap)
    {
        for (int join : joins)
        {
            drawEntityUsingVarioustrokeWidths(configuration, minArcWidth, maxArcWidth, minArcHeight, maxArcHeight, scales,
                    widths, eccentricities, dash1patterns, dash2patterns, cap, join);
        }
    }

    private void drawEntityUsingVarioustrokeWidths(GfxTestConfiguration configuration, int minArcWidth, int maxArcWidth,
            int minArcHeight, int maxArcHeight, float[] scales, float[] widths, float[] eccentricities,
            float[] dash1patterns, float[] dash2patterns, int cap, int join)
    {
        for (float width : widths)
        {
            drawEntityUsingVariousScales(configuration, minArcWidth, maxArcWidth, minArcHeight, maxArcHeight, scales,
                    eccentricities, dash1patterns, dash2patterns, cap, join, width);
        }
    }

    private void drawEntityUsingVariousScales(GfxTestConfiguration configuration, int minArcWidth, int maxArcWidth,
            int minArcHeight, int maxArcHeight, float[] scales, float[] eccentricities, float[] dash1patterns,
            float[] dash2patterns, int cap, int join, float width)
    {
        for (float scale : scales)
        {
            drawEntityUsingVariousEccentricities(configuration, minArcWidth, maxArcWidth, minArcHeight, maxArcHeight,
                    eccentricities, dash1patterns, dash2patterns, cap, join, width, scale);
        }
    }

    private void drawEntityUsingVariousEccentricities(GfxTestConfiguration configuration, int minArcWidth, int maxArcWidth,
            int minArcHeight, int maxArcHeight, float[] eccentricities, float[] dash1patterns, float[] dash2patterns,
            int cap, int join, float width, float scale)
    {
        for (float eccentricity : eccentricities)
        {
            drawEntityUsingVariousArcSizes(configuration, minArcWidth, maxArcWidth, minArcHeight, maxArcHeight,
                    dash1patterns, dash2patterns, cap, join, width, scale, eccentricity);
        }
    }

    private void drawEntityUsingVariousArcSizes(GfxTestConfiguration configuration, int minArcWidth, int maxArcWidth,
            int minArcHeight, int maxArcHeight, float[] dash1patterns, float[] dash2patterns, int cap, int join,
            float width, float scale, float eccentricity)
    {
        for (int arcHeight = minArcHeight; arcHeight <= maxArcHeight; arcHeight += this.arcStep)
        {
            for (int arcWidth = minArcWidth; arcWidth <= maxArcWidth; arcWidth += this.arcStep)
            {
                drawEntityUsingVariousDashPatterns(configuration, dash1patterns, dash2patterns, cap, join, width,
                        scale, eccentricity, arcHeight, arcWidth);
            }
        }
    }

    private void drawEntityUsingVariousDashPatterns(GfxTestConfiguration configuration, float[] dash1patterns,
            float[] dash2patterns, int cap, int join, float width, float scale, float eccentricity, int arcHeight,
            int arcWidth)
    {
        for (float dash1 : dash1patterns)
        {
            for (float dash2 : dash2patterns)
            {
                setStyleAndDrawEntity(configuration, cap, join, width, scale, eccentricity, arcHeight, arcWidth, dash1, dash2);
            }
        }
    }

    private void setStyleAndDrawEntity(GfxTestConfiguration configuration, int cap, int join, float width, float scale,
            float eccentricity, int arcHeight, int arcWidth, float dash1, float dash2)
    {
        EntityRenderingStyle entityRenderingStyle = new EntityRenderingStyle(cap,
                join, width, scale, dash1, dash2, arcHeight, arcWidth, false,
                eccentricity);
        performOneTest(configuration, this.currentTestNumber, entityRenderingStyle);
        this.currentTestNumber++;
    }

    @SuppressWarnings("boxing")
    private void performOneTest(GfxTestConfiguration configuration, int testNumber, EntityRenderingStyle entityRenderingStyle)
    {
        String testName = String.format("test%03d", testNumber); //$NON-NLS-1$
        this.log.logBegin(testName);
        int zoom = getZoom();
        TestImage image = new TestImage(configuration, zoom);
        Graphics2D graphics = image.getGraphics();
        logCurrentRenderingStyle(entityRenderingStyle);
        drawEntity(image, graphics, testNumber, entityRenderingStyle);
        graphics.dispose();
        try
        {
            writeZoomedImage(configuration, image, testName, zoom);
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
        this.log.logEnd(testName);
    }

    @SuppressWarnings({ "nls", "boxing" })
    private void logCurrentRenderingStyle(EntityRenderingStyle entityRenderingStyle)
    {
        this.log.logSet("cap", entityRenderingStyle.getCap());
        this.log.logSet("join", entityRenderingStyle.getJoin());
        this.log.logSet("width", entityRenderingStyle.getWidth());
        this.log.logSet("scale", entityRenderingStyle.getScale());
        float[] dash = entityRenderingStyle.getDash();
        this.log.logSet("dash", "[" + dash[0] + ", " + dash[1] + "]");
        this.log.logSet("arc width", entityRenderingStyle.getArcWidth());
        this.log.logSet("arc height", entityRenderingStyle.getArcHeight());
    }

    /**
     * Run test configured to use bitmap or postscript file as its output.
     * 
     * @param testType
     *            type of test - either RENDER_TEST or PRINT_TEST should be used
     *            at this moment
     * @param configuration
     *            configuration set by gfx-test harness for each test suite
     * @param method
     *            test method
     */
    private void tryToInvokeTestMethod(TestType testType, GfxTestConfiguration configuration, Method method)
    {
        // read method name
        String methodName = method.getName();
        // read test filter
        String filter = configuration.getFilter();

        // filter out test methods whose names don't contain given pattern
        if (filter != null)
        {
            // test name does not contains given pattern
            if (!methodName.contains(filter))
            {
                return;
            }
        }
        if (methodName.startsWith("test")) //$NON-NLS-1$
        {
            this.log.logBegin(methodName);
            switch (testType.value())
            {
                case RENDER_TEST:
                    tryToRunRenderTest(configuration, method, methodName);
                    break;
                case PRINT_TEST:
                    tryToRunPrintTest(configuration, method, methodName);
                    break;
            }
            this.log.logEnd(methodName);
        }
    }

    /**
     * Run test configured to use bitmap as its output.
     * 
     * @param configuration
     *            configuration set by gfx-test harness for each test suite
     * @param method
     *            test method
     * @param methodName
     *            name of test method
     */
    private void tryToRunRenderTest(GfxTestConfiguration configuration, Method method, String methodName)
    {
        int zoom = getZoom();
        TestImage image = new TestImage(configuration, zoom);
        Graphics2D gc = image.getGraphics();
        TestResult result = null;
        try
        {
            result = (TestResult) method.invoke(this, new Object[] { image, gc });
        }
        catch (Exception e)
        {
            e.printStackTrace();
            result = TestResult.ERROR;
        }
        finally
        {
            gc.dispose();
            logTestResult(result);
            try
            {
                writeZoomedImage(configuration, image, methodName, zoom);
            }
            catch (IOException e)
            {
                e.printStackTrace();
            }
            this.log.logEnd(methodName);
        }
    }

    /**
     * Run test configured to use printer as its output.
     * 
     * @param configuration
     *            configuration set by gfx-test harness for each test suite
     * @param method
     *            test method
     * @param methodName
     *            name of test method
     */
    private void tryToRunPrintTest(GfxTestConfiguration configuration, Method method, String methodName)
    {
        String psMimeType = DocFlavor.BYTE_ARRAY.POSTSCRIPT.getMimeType(); // only this works!
        File printOutputFile;
        File pngOutputFile;
        StreamPrintService streamPrintService;
        StreamPrintServiceFactory[] factories = PrinterJob.lookupStreamPrintServices(psMimeType);
        FileOutputStream fileOutputStream = null;
        if (factories.length == 0)
        {
            System.err.println("No factories found");
        }
        else
        {
            try
            {
                // prepare input and output files
                printOutputFile = constructOutputFile(methodName, "ps");
                pngOutputFile = constructOutputFile(methodName, "png");

                fileOutputStream = new FileOutputStream(printOutputFile);
                streamPrintService = factories[0].getPrintService(fileOutputStream);
                // streamPrintService can now be set as the service on a PrinterJob

                // set up initial print settings
                PrintRequestAttributeSet printAttributeSet = new HashPrintRequestAttributeSet();
                printAttributeSet.add(MediaSizeName.ISO_A4);
                printAttributeSet.add(new Copies(1));

                Doc doc = new SimpleDoc(new PrintingTest(this, method), DocFlavor.SERVICE_FORMATTED.PRINTABLE, null);
                try
                {
                    DocPrintJob docPrintJob = streamPrintService.createPrintJob(); 
                    PrintJobWatcher printJobWatcher = new PrintJobWatcher(docPrintJob);
                    this.log.logBegin(this.getClass().getName(), "printBegin " + methodName);
                    docPrintJob.print(doc, printAttributeSet);
                    // wait until print is done
                    printJobWatcher.waitForDone();
                    this.log.logEnd(this.getClass().getName(), "printDone " + methodName);
                }
                catch (PrintException e)
                {
                    e.printStackTrace();
                }
                finally
                {
                    // really ugly implementation :(
                    try
                    {
                        // must be sure that output stream is really closed
                        // because we are going to use output file with
                        // converter
                        fileOutputStream.flush();
                    }
                    catch (IOException e)
                    {
                        e.printStackTrace();
                    }
                    try
                    {
                        // we must be sure that output stream is really closed
                        // because we are going to use output file with
                        // converter
                        fileOutputStream.close();
                    }
                    catch (IOException e)
                    {
                        e.printStackTrace();
                    }
                    // convert PostScript file to a PNG using software RIP
                    PostScriptToPngConverter.convert(printOutputFile, pngOutputFile);
                }
            }
            catch (FileNotFoundException e)
            {
                e.printStackTrace();
            }
        }
    }

    /**
     * Creates an instance of File object which represents test result file, ie.
     * either PostScript file or PNG image.
     * 
     * @param methodName name of test method
     * @param suffix suffix, should be either "ps" or "png"
     * @return an instance of File object which represents test result file
     */
    private File constructOutputFile(String methodName, String suffix)
    {
        if ("ps".equals(suffix) || "png".equals(suffix))
        {
            return new File(this.suiteName + "_" + methodName + "." + suffix);
        }
        throw new RuntimeException("Bad suffix");
    }

    private void writeZoomedImage(GfxTestConfiguration configuration, TestImage sourceImage, String methodName, int zoom) throws IOException
    {
        if (zoom == 1)
        {
            sourceImage.writeImage(configuration, this.suiteName, methodName);
        }
        else
        {
            TestImage destinationImage = new TestImage(configuration, 1);
            zoomImage(sourceImage, zoom, destinationImage);
            drawGrid(destinationImage, zoom);
            destinationImage.writeImage(configuration, this.suiteName, methodName);
        }
    }

    private void zoomImage(TestImage sourceImage, int zoom, TestImage destinationImage)
    {
        for (int y = 0; y < destinationImage.getHeight(); y++)
        {
            for (int x = 0; x < destinationImage.getWidth(); x++)
            {
                destinationImage.setRGB(x, y, sourceImage.getRGB(x / zoom, y / zoom));
            }
        }
    }

    private void drawGrid(TestImage image, int zoom)
    {
        Graphics2D g = image.getGraphics();
        g.setColor(Color.GRAY);
        for (int x = 0; x < image.getWidth(); x += zoom)
        {
            g.drawLine(x, 0, x, image.getHeight() - 1);
        }
        for (int y = 0; y < image.getWidth(); y += zoom)
        {
            g.drawLine(0, y, image.getWidth() - 1, y);
        }
    }

    /**
     * Returns zoom set by annotation.
     *
     * @return actual zoom value
     */
    protected int getZoom()
    {
        Zoom zoom = this.getClass().getAnnotation(Zoom.class);
        return zoom == null ? 1 : zoom.value();
    }

    /**
     * Returns test type set by annotation.
     *
     * @return actual test type value
     */
    protected TestType getTestType()
    {
        return this.getClass().getAnnotation(TestType.class);
    }

    /**
     * Write test result to log file.
     *
     * @param result status of the last invoked test
     */
    protected void logTestResult(TestResult result)
    {
        String resultText = ""; //$NON-NLS-1$
        switch (result)
        {
        case PASSED:
            resultText = "$GREEN$PASSED"; //$NON-NLS-1$
            break;
        case FAILED:
            resultText = "$RED$FAILED"; //$NON-NLS-1$
            break;
        case ERROR:
            resultText = "$BLUE$ERROR"; //$NON-NLS-1$
            break;
        }
        this.log.logSet("test result", resultText); //$NON-NLS-1$
    }

    /**
     * Draw cross into graphics canvas, to mark shape's end points, curves's
     * control points etc. Size of cross is stored in constant CROSS_SIZE.
     * 
     * @param graphics
     *            graphics context for the image created by test case
     * @param x
     *            x-coordinate of the cross center
     * @param y
     *            y-coordinate of the cross center
     */
    public static void drawCross(Graphics2D graphics, int x, int y)
    {
        drawCross(graphics, x, y, CROSS_SIZE);
    }

    /**
     * Draw cross into graphics canvas, to mark shape's end points, curves's
     * control points etc.
     * 
     * @param graphics
     *            graphics context for the image created by test case
     * @param x
     *            x-coordinate of the cross center
     * @param y
     *            y-coordinate of the cross center
     * @param size
     *            size of cross (measured in pixels)
     */
    public static void drawCross(Graphics2D graphics, int x, int y, int size)
    {
        graphics.setStroke(new BasicStroke(1.0f));
        graphics.setColor(Color.RED);
        graphics.drawLine(x, y - size, x, y + size);
        graphics.drawLine(x - size, y, x + size, y);
    }

    /**
     * Return arc step.
     * 
     * @return arc step.
     */
	public int getArcStep()
	{
		return this.arcStep;
	}

    /**
     * Set arc step
     * 
     * @param arcStep
     *            new value of arc step.
     */
	public void setArcStep(int arcStep)
	{
		this.arcStep = arcStep;
	}

}
